import json
from lex.fluent.intent import Intent, DialogHookCode, ConfirmationPrompt, \
                              RejectionStatement, Prompt, FollowUpPrompt, \
                              ConclusionStatement, FulfillmentActivity, \
                              IntentSlot
from lex.fluent.slot import Slot, EnumerationValue
from utils import DictUtils
import unittest


class TestCase(unittest.TestCase):
    def test_can_set_name(self):
        i = Intent(Name='test')
        assert i.name == 'test'

    def test_can_set_description(self):
        i = Intent(Name='test', Description='description')
        assert i.description == 'description'

    def test_can_set_dialog_cook_hook(self):
        correct_j = {"name": "test", "description": "description", "dialogCookHook": {"uri":  "urihook", "messageVersion": "2"}}

        i = Intent(Name='test', Description='description')
        i.with_dialog_hook(DialogHookCode(Uri='urihook',
                                          MessageVersion='2'))
        assert DictUtils.are_same(i.to_json(), correct_j)

    def test_can_add_slot(self):
        correct_j = {'name': 'test', 'description': 'description', 'slots': [{'name': 'test', 'createVersion': 'True', 'valueSelectionStrategy': 'ORIGINAL_VALUE', 'description': 'description', 'enumerationValues': [{'value': 'AMAZON.STRING', 'synonyms': ['test', 'mytest']}], 'checksum': 'test_chk'}]}
        s = Slot()
        s.with_name('test') \
         .with_description('description') \
         .with_checksum('test_chk') \
         .with_create_version('True') \
         .with_enumeration_value(EnumerationValue(Value='AMAZON.STRING', Synonyms=['test', 'mytest']))
        i = Intent(Name='test', Description='description')
        i.with_slot(s)
        assert DictUtils.are_same(i.to_json(), correct_j)

    def test_can_add_sample_utterances(self):
        correct_j = {'name': 'test', 'description': 'description', 'sampleUtterances': [['what time', 'which time']]}
        i = Intent(Name='test', Description='description')
        i.with_sample_utterances(['what time', 'which time'])

        assert DictUtils.are_same(i.to_json(), correct_j)

    def test_can_add_confirmation_prompt(self):
        correct_j = {'name': 'test', 'description': 'description', 'confirmationPrompt': {'messages': [{'contentType': 'PlainText', 'content': 'content'}, {'contentType': 'SSML', 'content': 'ssml_content', 'groupNumber': 123}], 'maxAttempts': 3, 'responseCard': 'test_card'}}
        i = Intent(Name='test', Description='description')
        c = ConfirmationPrompt()
        c.with_message('PlainText', 'content') \
         .with_message('SSML', 'ssml_content', 123) \
         .with_max_attempts(3) \
         .with_response_card('test_card')
        i.with_confirmation_prompt(c)
        assert DictUtils.are_same(i.to_json(), correct_j)

    def test_can_add_rejection_statement(self):
        correct_j = {'name': 'test', 'description': 'description', 'rejectionStatement': {'messages': [{'contentType': 'PlainText', 'content': 'content'}, {'contentType': 'SSML', 'content': 'ssml_content', 'groupNumber': 123}], 'responseCard': 'test_card'}}
        i = Intent(Name='test', Description='description')
        r = RejectionStatement()
        r.with_message('PlainText', 'content') \
         .with_message('SSML', 'ssml_content', 123) \
         .with_response_card('test_card')
        i.with_rejection_statement(r)
        print(i.to_json())
        assert DictUtils.are_same(i.to_json(), correct_j)

    def test_can_add_follow_up_prompt(self):
        correct_j = {'name': 'test', 'description': 'description', 'followUpPrompt': {'prompt': {'messages': [{'contentType': 'PlainText', 'content': 'content'}, {'contentType': 'SSML', 'content': 'ssml_content', 'groupNumber': 123}], 'maxAttempts': 3, 'responseCard': 'test_card'}, 'rejection_statement': {'messages': [{'contentType': 'PlainText', 'content': 'content'}, {'contentType': 'SSML', 'content': 'ssml_content', 'groupNumber': 123}], 'responseCard': 'test_card'}}}
        i = Intent(Name='test', Description='description')

        p = Prompt()
        p.with_message('PlainText', 'content') \
         .with_message('SSML', 'ssml_content', 123) \
         .with_max_attempts(3) \
         .with_response_card('test_card')

        r = RejectionStatement()
        r.with_message('PlainText', 'content') \
         .with_message('SSML', 'ssml_content', 123) \
         .with_response_card('test_card')
        f = FollowUpPrompt()
        f.with_rejection_statement(r) \
         .with_prompt(p)
        i.with_follow_up_prompt(f)
        print(i.to_json())
        assert DictUtils.are_same(i.to_json(), correct_j)

    def test_can_add_conclusion_statement(self):
        correct_j = {'name': 'test', 'description': 'description', 'conclusionStatement': {'messages': [{'contentType': 'PlainText', 'content': 'content'}, {'contentType': 'SSML', 'content': 'ssml_content', 'groupNumber': 123}], 'responseCard': 'test_card'}}
        i = Intent(Name='test', Description='description')
        c = ConclusionStatement()
        c.with_message('PlainText', 'content') \
         .with_message('SSML', 'ssml_content', 123) \
         .with_response_card('test_card')
        i.with_conclusion_statement(c)
        print(i.to_json())
        assert DictUtils.are_same(i.to_json(), correct_j)

    def test_can_set_fulfillment_activity(self):
        correct_j = {'name': 'test', 'description': 'description', 'fulfillmentActivity': {'codeHook': {'uri': 'urihook', 'messageVersion': '2'}, 'type': 'ReturnIntent'}}

        i = Intent(Name='test', Description='description')
        f = FulfillmentActivity()
        f.with_code_hook(DialogHookCode(Uri='urihook',
                                          MessageVersion='2'))
        f.with_type('ReturnIntent')
        i.with_fulfillment_activity(f)
        assert DictUtils.are_same(i.to_json(), correct_j)

    def test_can_pass_slot_to_intent_slot(self):
        s = Slot(Name='testslot', Description='testdescript')
        intent_slot = IntentSlot(Slot=s).with_slot_constraint('Required')
        assert intent_slot.description == 'testdescript'
        assert intent_slot.name == 'testslot'
        assert intent_slot.slot_constraint == 'Required'



from lex.bots import BaseBot
import time


class PollexyHygieneBot(BaseBot):
    def __init__(self, lexbot):
        self.bot_name = 'PollexyHygieneBot'
        self.lexbot = lexbot
        super(PollexyHygieneBot, self).__init__()

    def on_fulfilled(self):
        print(self.lexbot.last_intent)
        if self.lexbot.last_intent == 'PollexyBrushTeethIntent':
            self.lexbot.output(
                Message="Great! Let's go! Pick up your toothbrush." +
                        "Put some toothpaste on the toothbrush.")
            self.lexbot.next_intent = \
                'I want to put toothbrush on my toothpaste'

        if self.lexbot.last_intent == 'PollexyPutToothpasteIntent':
            self.lexbot.output(
                Message="Great! Let's go! Pick up your toothbrush.")
            self.lexbot.next_intent = \
                'brush my top teeth'

        if self.lexbot.last_intent == 'PollexyBrushTopTeethIntent':
            self.lexbot.output(
                Message="Awesome. Start brushing your top teeth. I'll " +
                        "tell you when to switch. Start brushing your top " +
                        "teeth now.")
            time.sleep(10)
            self.lexbot.output(
                Message="OK, let's switch to your bottom teeth. Start " +
                        "brushing your bottom teeth now.")
            time.sleep(10)
            self.lexbot.output(
                Message="Great, you're all done brushing your teeth!")
            self.lexbot.next_intent = ""

        super(PollexyHygieneBot, self).on_fulfilled()

    def on_failed(self):
        intent = self.lexbot.last_response['intentName']
        if intent == 'PollexyBrushTeethIntent':
            self.lexbot.next_intent = 'I want to brush my teeth'
            self.lexbot.output(Message="OK, I'll wait a moment for you.")
            time.sleep(5)
        super(PollexyHygieneBot, self).on_failed()

    def on_transition_in(self):
        pass

    def on_transition_out(self):
        pass

    def on_cancel(self):
        pass

    def on_needs_intent(self):
        pass

    def on_response(self):
        pass

    def register(self):
        super(PollexyHygieneBot, self).register()



# Copyright 2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Amazon Software License (the "License"). You may not
# use this file except in compliance with the License. A copy of the
# License is located at:
#    http://aws.amazon.com/asl/
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, expressi
# or implied. See the License for the specific language governing permissions
# and limitations under the License.

import boto3
from contextlib import closing
import tempfile
import os
import logging
import sys
with open(os.devnull, 'w') as f:
    # disable stdout
    oldstdout = sys.stdout
    sys.stdout = f

    import pygame

    # enable stdout
    sys.stdout = oldstdout


class Speaker(object):
    def __init__(self, **kwargs):
        self.voice = kwargs.pop("VoiceId", "Joanna")
        self.output_format = kwargs.pop("OutputFormat", "ogg_vorbis")
        self.chime_path = kwargs.pop("ChimePath",
                                     os.path.expanduser("~/.pollexy/.cache/chimes"))
        self.no_audio = kwargs.pop("NoAudio", False)
        self.is_audio_ready = False
        self.audio_file_path = ""

    def just_say(self, **kwargs):
        include_chime = kwargs.pop('IncludeChime', False)
        message = kwargs.pop('Message', '')
        voice_id = kwargs.pop('VoiceId', 'Joanna')
        self.generate_audio(Message=message, VoiceId=voice_id)
        self.speak(IncludeChime=include_chime)

    def generate_audio(self, **kwargs):
        message = kwargs.pop("Message", "")
        text_type = kwargs.pop("TextType", "text")
        voice = kwargs.pop("VoiceId", self.voice).capitalize()
        if not message:
            return
        if not self.no_audio:
            polly = boto3.client('polly')
            response = polly.synthesize_speech(
                Text=message,
                OutputFormat="ogg_vorbis",
                TextType=text_type,
                VoiceId=voice)

            fd, path = tempfile.mkstemp(suffix=".ogg")

            with closing(response["AudioStream"]) as stream:
                with open(path, 'wb') as file:
                    file.write(stream.read())

            self.audio_file_path = path
        self.message = message
        self.is_audio_ready = True

    def speak(self, **kwargs):
        include_chime = kwargs.get('IncludeChime', False)
        chime = kwargs.pop("Chime", "three_tone_chime")
        if not self.is_audio_ready:
            print("Nothing to say . . . call generate_audio() first")
            return
        if self.no_audio:
            print("No audio . . . speech would be:\n%s" % self.message)
            return "audio: %s"
        else:
            audio_file_to_play = kwargs.pop("AudioFilePath",
                                            self.audio_file_path)
            pygame.mixer.pre_init(44100, -16, 2, 2048)
            pygame.init()
            pygame.mixer.init()
            if (include_chime):
                logging.info('chime={}/{}.wav'.format(self.chime_path, chime))
                pygame.mixer.music.load("%s/%s.wav" % (self.chime_path, chime))
                pygame.mixer.music.play()
            while pygame.mixer.music.get_busy():
                pygame.time.Clock().tick(10)
            pygame.mixer.music.load(audio_file_to_play)
            pygame.mixer.music.play(1)
            while pygame.mixer.music.get_busy():
                pygame.time.Clock().tick(10)

    def cleanup(self):
        if (self.audio_file_path):
            os.unlink(self.audio_file_path)
            self.audio_file_path = ""
        self.message = ""
        self.is_audio_ready = False
